import statsmodels.api as sm
import pandas as pd 
import xlwings as xw
import numpy as np
import pyttsx3]

def get_column_letter(num):
    letters = ''
    while num:
        mod = (num - 1) % 26
        letters += chr(mod + 65)
        num = (num - 1) // 26
    return ''.join(reversed(letters))
   

def fx(x,pwr):
  if pwr > 0:
    return x**pwr
  elif pwr < 0:
    return 1/x**abs(pwr)
  else:
    return np.log(x)

def calcColumns(sheet):
  col = 0
  while 1:
    col += 1
    if sheet.range(1,col).value is None:
      break
  return col-1

e = pyttsx3.init()
rate = e.getProperty('rate')
e.setProperty('rate', rate-50)

# Load your data (replace with your actual data)
wb = xw.Book('bestmlr1.xlsx')
sheetData = wb.sheets['Data']
sheetTransf = wb.sheets['Transf']
sheetRes = wb.sheets['Results']
sheetList = wb.sheets['List']
sheetYt = wb.sheets['Yt']
sheetXt = wb.sheets['Xt']
# clear sheets
sheetList.range('A2:Z10000').value = ""
sheetYt.range('A1:Z10000').value = ""
sheetXt.range('A1:Z10000').value = ""
row = 1
while 1:
  row += 1
  if sheetData.range(row,1).value is None:
    break
maxrows = row - 1

yPwrs = sheetTransf.range("A2:A4").value
xPwrs = sheetTransf.range("B2:B4").value
minAdjR2 = sheetTransf.range("C2").value

yCols = 0
for yPwr in np.arange(yPwrs[0],yPwrs[2]+yPwrs[1],yPwrs[1]):
  yCols += 1
  for row in range(2,maxrows+1):
    sheetYt.range(row-1,yCols).value = fx(sheetData.range(row,1).value,yPwr)

xCols = 0
for xPwr in np.arange(xPwrs[0],xPwrs[2]+xPwrs[1],xPwrs[1]):
  xCols += 1
  for row in range(2,maxrows+1):
    sheetXt.range(row-1,xCols).value = fx(sheetData.range(row,2).value,xPwr)


bestAdjR2 = 0
gRow = 1

e.say("Initialization Done")
e.runAndWait()

yCols = 0
for yPwr in np.arange(yPwrs[0],yPwrs[2]+yPwrs[1],yPwrs[1]):
  yCols += 1
  s = get_column_letter(yCols)
  s2 = s + "1:" + s + str(maxrows-1)
  Ysr = sheetYt.range(s2)
  Ysr.copy()
  Ydr = sheetData.range("C2")
  Ydr.paste()

  xCols = 0  
  for xPwr in np.arange(xPwrs[0],xPwrs[2]+xPwrs[1],xPwrs[1]):
    xCols += 1
    s = get_column_letter(xCols)
    s2 = s + "1:" + s + str(maxrows-1)
    Xsr = sheetXt.range(s2)
    Xsr.copy()
    Xdr = sheetData.range("D2")
    Xdr.paste()
    
    data = sheetData.range((2,3),(maxrows,4)).value
    
    # Convert data to a pandas DataFrame
    df = pd.DataFrame(data)
    df.columns = ['Yt','Xt']
    # Define dependent and independent variables
    y = df['Yt']
    X = df['Xt']
    # Add a constant to the independent variables (for the intercept term)
    X = sm.add_constant(X)

    # Fit the model
    model = sm.OLS(y, X).fit()
    
    if model.rsquared_adj > bestAdjR2:
      bestAdjR2 = model.rsquared_adj
      bestYpwr = yPwr
      bestXpwr = xPwr
      bestModel = model

    if model.rsquared_adj >= minAdjR2:
      gRow += 1
      sheetList.range((gRow,1)).value = model.rsquared_adj
      sheetList.range((gRow,2)).value = yPwr
      sheetList.range((gRow,3)).value = xPwr
      sheetList.range((gRow,4)).value= model.params.iloc[0]
      sheetList.range((gRow,5)).value= model.params.iloc[1]

model = bestModel

# Print the summary of the best regression results
print(model.summary())
print("Best Y power", bestYpwr)
print("Best X power", bestXpwr)

sheetRes.range("B1").value = model.rsquared_adj
sheetRes.range("B2").value = model.fvalue
sheetRes.range("B3").value = model.f_pvalue
sheetRes.range("B4").value = model.aic
sheetRes.range("D2").value = bestYpwr
sheetRes.range("F2").value = bestXpwr
sheetRes.range("E3").value = model.params.iloc[0]
sheetRes.range("F3").value = model.params.iloc[1]
sheetRes.range("E4").value = model.pvalues.iloc[0]
sheetRes.range("F4").value = model.pvalues.iloc[1]

e.say("Calculations Done")
e.runAndWait()